using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Runtime.CompilerServices;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using Newtonsoft.Json;
using Newtonsoft.Json.Serialization;
using Ninject;
using TourDataManager.Entities;

namespace TourDataManager {
    
    public interface ITourFilesListFetcher {
        Task<File[]> FetchFilesAsync(long tourId);
    }
    
    public class FetchTourContentUseCase : ITourFilesListFetcher{
        
        [Inject] public HttpClient HttpClient{ get; set; }
        [Inject] public Db Database{ get; set; }

        public ContentLoadingTask FetchTourContentFromServerAsync(long tourId){
            Debug.Log($"ContentLoadingTask queued for tour {tourId}");
            var contentLoadingTask = new ContentLoadingTask(tourId, this);
            // add to queue
            return contentLoadingTask;
        }

        /// <summary>
        ///  Получить список файлов
        /// </summary>
        /// <param name="tourId"></param>
        /// <returns></returns>
        public async Task<File[]> FetchFilesAsync(long tourId){
            Debug.Log("Fetching Files list");
            var response = await HttpClient.GetAsync(
                $"https://biganto.com/api-novus/tours.getFiles?client=desktopplayer&client_version=3.0&v=2.0&id={tourId}");
            var content = await response.Content.ReadAsStringAsync();
            var fukin_dict_wrap = JsonConvert.DeserializeObject<Dictionary<string,File[]>>(content);
            var files = fukin_dict_wrap[tourId.ToString()];
            
            var regex = new Regex(@"assets.+?(?=\?|$)");
            // TODO Что если не сматчится?
            foreach (var file in files){
                // Id AutoIncrement
                var val = regex.Match(file.Url).Value;
                System.Diagnostics.Debug.Assert(!string.IsNullOrEmpty(val));
                file.LocalUrl = TourDataManager.GetPathInPersistent(val);
            }
            Database.InsertFiles(files);
            return files;
        }
    }
}